@@ -25,7 +25,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-file-chooser-dialog.c 5646 2007-06-24 19:56:01Z pborelli $
+ * $Id: gedit-file-chooser-dialog.c 5659 2007-06-28 08:36:12Z pborelli $
  */
 
 /* TODO: Override set_extra_widget */
@@ -162,11 +162,11 @@ all_text_files_filter (const GtkFileFilterInfo *filter_info,
 	if (known_mime_types == NULL)
 	{
 		GtkSourceLanguageManager *lm;
-		const GSList *languages;
-		const GSList *l;
+		GSList *languages;
+		GSList *l;
 
 		lm = gedit_get_language_manager ();
-		languages = gtk_source_language_manager_get_available_languages (lm);
+		languages = gtk_source_language_manager_list_languages (lm);
 
 		for (l = languages; l != NULL; l = l->next)
 		{
@@ -201,6 +201,8 @@ all_text_files_filter (const GtkFileFilterInfo *filter_info,
 			g_strfreev (mime_types);
 		}
 
+		g_slist_free (languages);
+
 		/* known_mime_types always has "text/plain" as first item" */
 		known_mime_types = g_slist_prepend (known_mime_types, g_strdup ("text/plain"));
 	}
