@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c 5428 2007-01-08 22:02:42Z pborelli $
+ * $Id: gedit-window.c 5663 2007-06-29 08:52:15Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,8 +37,6 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
-#include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagesmanager.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "gedit-ui.h"
@@ -50,7 +48,7 @@
 #include "gedit-utils.h"
 #include "gedit-commands.h"
 #include "gedit-debug.h"
-#include "gedit-languages-manager.h"
+#include "gedit-language-manager.h"
 #include "gedit-prefs-manager-app.h"
 #include "gedit-panel.h"
 #include "gedit-documents-panel.h"
@@ -801,14 +799,15 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 	gedit_plugins_engine_update_plugins_ui (window, FALSE);
 }
 
+#define GEDIT_SOURCE_LANGUAGE_KEY "sourcelanguage"
+
 static void
 language_toggled (GtkToggleAction *action,
 		  GeditWindow     *window)
 {
 	GeditDocument *doc;
-	const GSList *languages;
-	const GtkSourceLanguage *lang;
-	gint n;
+	GtkSourceLanguage *lang;
+	const gchar *lang_id;
 
 	if (gtk_toggle_action_get_active (action) == FALSE)
 		return;
@@ -817,26 +816,29 @@ language_toggled (GtkToggleAction *action,
 	if (doc == NULL)
 		return;
 
-	n = gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action));
-
-	if (n < 0)
+	lang_id = g_object_get_data (G_OBJECT (action),
+				     GEDIT_SOURCE_LANGUAGE_KEY);
+	if (lang_id == NULL)
 	{
 		/* Normal (no highlighting) */
 		lang = NULL;
 	}
 	else
 	{
-		languages = gedit_languages_manager_get_available_languages_sorted (
-						gedit_get_languages_manager ());
-
-		lang = GTK_SOURCE_LANGUAGE (g_slist_nth_data ((GSList *) languages, n));
+		lang = gtk_source_language_manager_get_language_by_id (
+				gedit_get_language_manager (),
+				lang_id);
+		if (lang == NULL)
+		{
+			g_warning ("Could not get language %s\n", lang_id);
+		}
 	}
 
-	gedit_document_set_language (doc, (GtkSourceLanguage *) lang);
+	gedit_document_set_language (doc, lang);
 }
 
 static gchar *
-escape_section_name (gchar *name)
+escape_section_name (const gchar *name)
 {
 	gchar *tmp;
 	gchar *ret;
@@ -863,9 +865,10 @@ create_language_menu_item (GtkSourceLanguage *lang,
 	GtkRadioAction *action;
 	GtkAction *normal_action;
 	GSList *group;
-	gchar *section;
+	const gchar *section;
 	gchar *escaped_section;
-	gchar *lang_name;
+	const gchar *lang_id;
+	const gchar *lang_name;
 	gchar *escaped_lang_name;
 	gchar *tip;
 	gchar *path;
@@ -898,7 +901,6 @@ create_language_menu_item (GtkSourceLanguage *lang,
 
 	/* now add the language item to the section */
 	lang_name = gtk_source_language_get_name (lang);
-
 	escaped_lang_name = g_markup_escape_text (lang_name, -1);
 
 	tip = g_strdup_printf (_("Use %s highlight mode"), lang_name);
@@ -911,6 +913,13 @@ create_language_menu_item (GtkSourceLanguage *lang,
 				       NULL,
 				       index);
 
+	/* store lang id */
+	lang_id = gtk_source_language_get_id (lang);
+	g_object_set_data_full (G_OBJECT (action),
+				GEDIT_SOURCE_LANGUAGE_KEY,
+				g_strdup (lang_id),
+				(GDestroyNotify) g_free);
+
 	/* Action is added with a NULL accel to make the accel overridable */
 	gtk_action_group_add_action_with_accel (window->priv->languages_action_group,
 						GTK_ACTION (action),
@@ -937,9 +946,7 @@ create_language_menu_item (GtkSourceLanguage *lang,
 
 	g_free (path);
 	g_free (tip);
-	g_free (lang_name);
 	g_free (escaped_lang_name);
-	g_free (section);
 	g_free (escaped_section);
 }
 
@@ -947,8 +954,8 @@ static void
 create_languages_menu (GeditWindow *window)
 {
 	GtkRadioAction *action_none;
-	const GSList *languages;
-	const GSList *l;
+	GSList *languages;
+	GSList *l;
 	guint id;
 	gint i;
 
@@ -984,14 +991,18 @@ create_languages_menu (GeditWindow *window)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_none), TRUE);
 
 	/* now add all the known languages */
-	languages = gedit_languages_manager_get_available_languages_sorted (
-						gedit_get_languages_manager ());
+	languages = gedit_language_manager_list_languages_sorted (
+						gedit_get_language_manager ());
 
 	for (l = languages, i = 0; l != NULL; l = l->next, ++i)
-		create_language_menu_item (GTK_SOURCE_LANGUAGE (l->data),
-					    i,
-					    id,
-					    window);
+	{
+		create_language_menu_item (l->data,
+					   i,
+					   id,
+					   window);
+	}
+
+	g_slist_free (languages);
 }
 
 static void
@@ -1002,7 +1013,7 @@ update_languages_menu (GeditWindow *window)
 	GList *l;
 	GtkAction *action;
 	GtkSourceLanguage *lang;
-	gchar *lang_name;
+	const gchar *lang_name;
 	gchar *escaped_lang_name;
 
 	doc = gedit_window_get_active_document (window);
@@ -1013,7 +1024,7 @@ update_languages_menu (GeditWindow *window)
 	if (lang != NULL)
 		lang_name = gtk_source_language_get_name (lang);
 	else
-		lang_name = g_strdup ("LangNone");
+		lang_name = "LangNone";
 
 	escaped_lang_name = g_markup_escape_text (lang_name, -1);
 
@@ -1040,7 +1051,6 @@ update_languages_menu (GeditWindow *window)
 	}
 
 	g_list_free (actions);
-	g_free (lang_name);
 	g_free (escaped_lang_name);
 }
 
@@ -2431,32 +2441,38 @@ can_search_again (GeditDocument *doc,
 
 static void
 can_undo (GeditDocument *doc,
-          gboolean       can,
-          GeditWindow   *window)
+	  GParamSpec    *pspec,
+	  GeditWindow   *window)
 {
 	GtkAction *action;
-	
+	gboolean sensitive;
+
+	sensitive = gtk_source_buffer_can_undo (GTK_SOURCE_BUFFER (doc));
+
 	if (doc != gedit_window_get_active_document (window))
 		return;
-		
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					     "EditUndo");
-	gtk_action_set_sensitive (action, can);
+	gtk_action_set_sensitive (action, sensitive);
 }
 
 static void
 can_redo (GeditDocument *doc,
-          gboolean       can,
-          GeditWindow   *window)
+	  GParamSpec    *pspec,
+	  GeditWindow   *window)
 {
 	GtkAction *action;
+	gboolean sensitive;
+
+	sensitive = gtk_source_buffer_can_redo (GTK_SOURCE_BUFFER (doc));
 
 	if (doc != gedit_window_get_active_document (window))
 		return;
-	
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					     "EditRedo");
-	gtk_action_set_sensitive (action, can);
+	gtk_action_set_sensitive (action, sensitive);
 }
 
 static void
@@ -2505,76 +2521,6 @@ sync_languages_menu (GeditDocument *doc,
 	update_languages_menu (window);
 }
 
-static void
-update_default_path (GeditWindow   *window,
-		     GeditDocument *doc)
-{
-	gchar *uri;
-
-	uri = gedit_document_get_uri (doc);
-	// CHECK: what does it happens when loading from stdin? - Paolo
-	g_return_if_fail (uri != NULL);
-
-	if (gedit_utils_uri_has_file_scheme (uri))
-	{
-		gchar *default_path;
-
-		// CHECK: does it work with uri chaining? - Paolo
-		default_path = g_path_get_dirname (uri);
-
-		g_return_if_fail (strlen (default_path) >= 5 /* strlen ("file:") */);
-		if (strcmp (default_path, "file:") == 0)
-		{
-			g_free (default_path);
-		
-			default_path = g_strdup ("file:///");
-		}
-
-		g_free (window->priv->default_path);
-
-		window->priv->default_path = default_path;
-
-		gedit_debug_message (DEBUG_WINDOW, 
-				     "New default path: %s", default_path);
-	}
-
-	g_free (uri);
-}
-
-static void
-doc_loaded (GeditDocument *doc,
-	    const GError  *error,
-	    GeditWindow   *window)
-{
-	if (error != NULL)
-	{
-		gedit_debug_message (DEBUG_WINDOW, "Error");
-		return;
-	}
-	
-	update_default_path (window, doc);
-}
-
-static void
-doc_saved (GeditDocument *doc,
-	   const GError  *error,
-	   GeditWindow   *window)
-{
-	if (error != NULL)
-	{
-		gedit_debug_message (DEBUG_WINDOW, "Error");
-		return;
-	}
-	
-	if (!_gedit_document_is_saving_as (doc))
-	{
-		gedit_debug_message (DEBUG_WINDOW, "Not saving as");
-		return;
-	}
-
-	update_default_path (window, doc);
-}
-
 static void
 editable_changed (GeditView  *view,
                   GParamSpec  *arg1,
@@ -2633,11 +2579,11 @@ notebook_tab_added (GeditNotebook *notebook,
 			  G_CALLBACK (can_search_again),
 			  window);
 	g_signal_connect (doc,
-			  "can-undo",
+			  "notify::can-undo",
 			  G_CALLBACK (can_undo),
 			  window);
 	g_signal_connect (doc,
-			  "can-redo",
+			  "notify::can-redo",
 			  G_CALLBACK (can_redo),
 			  window);
 	g_signal_connect (doc,
@@ -2648,14 +2594,6 @@ notebook_tab_added (GeditNotebook *notebook,
 			  "notify::language",
 			  G_CALLBACK (sync_languages_menu),
 			  window);
-	g_signal_connect (doc,
-			  "loaded",
-			  G_CALLBACK (doc_loaded),
-			  window);			  			  
-	g_signal_connect (doc,
-			  "saved",
-			  G_CALLBACK (doc_saved),
-			  window);
 	g_signal_connect (view,
 			  "toggle_overwrite",
 			  G_CALLBACK (update_overwrite_mode_statusbar),
@@ -2705,16 +2643,16 @@ notebook_tab_removed (GeditNotebook *notebook,
 	GeditView     *view;
 	GeditDocument *doc;
 	GtkAction     *action;
-	
+
 	gedit_debug (DEBUG_WINDOW);
-	
+
 	g_return_if_fail ((window->priv->state & GEDIT_WINDOW_STATE_SAVING_SESSION) == 0);
-		
+
 	--window->priv->num_tabs;
-	
+
 	view = gedit_tab_get_view (tab);
 	doc = gedit_tab_get_document (tab);
-	
+
 	g_signal_handlers_disconnect_by_func (tab,
 					      G_CALLBACK (sync_name), 
 					      window);
@@ -2739,12 +2677,6 @@ notebook_tab_removed (GeditNotebook *notebook,
 	g_signal_handlers_disconnect_by_func (doc,
 					      G_CALLBACK (sync_languages_menu),
 					      window);
-	g_signal_handlers_disconnect_by_func (doc, 
-					      G_CALLBACK (doc_loaded),
-					      window);
-	g_signal_handlers_disconnect_by_func (doc, 
-					      G_CALLBACK (doc_saved),
-					      window);					      				      
 	g_signal_handlers_disconnect_by_func (view, 
 					      G_CALLBACK (update_overwrite_mode_statusbar),
 					      window);
@@ -3666,6 +3598,36 @@ _gedit_window_get_default_path (GeditWindow *window)
 	return window->priv->default_path;
 }
 
+void
+_gedit_window_set_default_path (GeditWindow *window,
+				const gchar *uri)
+{
+	g_return_if_fail (uri != NULL);
+
+	if (gedit_utils_uri_has_file_scheme (uri))
+	{
+		gchar *default_path;
+
+		// CHECK: does it work with uri chaining? - Paolo
+		default_path = g_path_get_dirname (uri);
+
+		g_return_if_fail (strlen (default_path) >= 5 /* strlen ("file:") */);
+		if (strcmp (default_path, "file:") == 0)
+		{
+			g_free (default_path);
+		
+			default_path = g_strdup ("file:///");
+		}
+
+		g_free (window->priv->default_path);
+
+		window->priv->default_path = default_path;
+
+		gedit_debug_message (DEBUG_WINDOW, 
+				     "New default path: %s", default_path);
+	}
+}
+
 /* Returns the documents that need to be saved before closing the window */
 GList *
 gedit_window_get_unsaved_documents (GeditWindow *window)
